/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.linklist;

import org.tinygroup.commons.processor.Processor;

/**
 * Created by IntelliJ IDEA. User: luoguo Date: 11-4-1 Time: 下午9:02 To change
 * this template use File | Settings | File Templates.
 */
public interface LinkList<E> {

	/**
	 * 将指定元素插入到此链表的开头 返回所插入的元素
	 * 
	 * @param e
	 * @return
	 */
	E addFirst(E e);

	/**
	 * 将指定元素插入到此链表的结尾 返回所插入的元素
	 * 
	 * @param e
	 * @return
	 */
	E addLast(E e);

	/**
	 * 获取链表的第一个元素
	 * 
	 * @return
	 */
	E getFirst();

	/**
	 * 获取链表的最后一个元素
	 * 
	 * @return
	 */
	E getLast();

	/**
	 * 删除链表的第一个元素 返回所删除的元素
	 * 
	 * @return
	 */
	E removeFirst();

	/**
	 * 删除链表的最后一个元素 返回所删除的元素
	 * 
	 * @return
	 */
	E removeLast();

	/**
	 * 删除指定的元素 返回所删除的元素
	 * 
	 * @param e
	 * @return
	 * @throws NoElementException
	 */
	E remove(E e);

	/**
	 * 在指定元素前添加元素 返回所添加的元素
	 * 
	 * @param find
	 * @param e
	 * @return
	 * @throws NoElementException
	 */
	E addBefore(E find, E e);

	/**
	 * 在指定元素后添加元素 返回所添加的元素
	 * 
	 * @param find
	 * @param e
	 * @return
	 * @throws NoElementException
	 */
	E addAfter(E find, E e);

	/**
	 * 从些链表中清除所有的元素
	 */
	void clear();

	/**
	 * 获取链表的元素个数
	 * 
	 * @return
	 */
	int size();

	/**
	 * 遍历链表的所有元素 使每个元素都作为IProcess接口内process()方法的参数
	 * 
	 * @param process
	 */
	void foreach(Processor<E> process);

}
